"""Linton 'init' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import os
import shutil

import importlib_resources

from linton.warnings_util import die


def run(args: argparse.Namespace) -> None:
    """'init' command handler"""
    # Check directory does not exist, unless we used --force
    if not args.force and os.path.exists(args.directory):
        die(f"output {args.directory} already exists")

    # Copy the demo files to the new project
    with importlib_resources.as_file(importlib_resources.files()) as fspath:
        shutil.copytree(
            os.path.join(fspath, "..", "init-pages"),
            args.directory,
            # See https://github.com/rrthomas/linton/issues/7
            ignore=shutil.ignore_patterns("__pycache__"),
            dirs_exist_ok=args.force,
        )


def add_subparser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "init",
        help="create a new Linton project",
    )
    parser.add_argument(
        "--force",
        help="overwrite existing files and directories",
        action="store_true",
    )
    parser.add_argument("directory", metavar="DIRECTORY", help="output directory")
    parser.set_defaults(func=run)



"""'linton init' tests.

Copyright (c) Reuben Thomas 2024.
Released under the GPL version 3, or (at your option) any later version.
"""

from collections.abc import Callable
from pathlib import Path

from pytest import CaptureFixture
from testutils import Case, dir_test, make_tests

from linton import main


pytestmark = make_tests(
    main,
    Path(__file__).parent.resolve() / "test-files",
    Case(
        "default",
        ["init"],
    ),
)


def test_init(
    function: Callable[[list[str]], None],
    case: Case,
    fixture_dir: Path,
    capsys: CaptureFixture[str],
    regenerate_expected: bool,
) -> None:
    dir_test(
        function,
        case,
        fixture_dir,
        capsys,
        regenerate_expected,
    )



"""Linton 'publish' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024-2025
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import os
import subprocess
import sys

from linton.warnings_util import die


def run(args: argparse.Namespace) -> None:
    """'publish' command handler"""
    cmd = ["nancy"]

    # Deal with --update
    if args.update:
        args.force = True
        cmd.append("--update")

    # Check output either does not exist, or is an empty directory, unless
    # --force given
    if os.path.exists(args.output) and not (
        os.path.isdir(args.output) and len(os.listdir(args.output)) == 0
    ):
        if args.force:
            cmd.append("--delete")
        else:
            die(f"output {args.output} exists and is not an empty directory")

    try:
        # Render the project files to the output
        subprocess.check_output(cmd + [args.document_root, args.output])

        # Check links unless disabled
        output_dir = args.output
        if not output_dir.endswith("/"):
            output_dir += "/"
        if not args.no_check_links:
            subprocess.check_call(["linkchecker", output_dir])
    except subprocess.CalledProcessError as err:
        if err.stderr is not None:
            print(err.stderr.decode("iso-8859-1"), file=sys.stderr)
        die(f"Error code {err.returncode} running: {' '.join(map(str, err.cmd))}")


def add_subparser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "publish",
        help="convert a directory of Markdown files and other resources into a web site",
        epilog="The output DIRECTORY cannot be a subdirectory of the input directory.",
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="overwrite output directory even if it is not empty",
    )
    parser.add_argument(
        "-u",
        "--update",
        action="store_true",
        help="update an existing site, writing only changed files (implies --force)",
    )
    parser.add_argument(
        "--base-url",
        metavar="URL",
        help="base URL of web site relative to root of server [default: %(default)s]",
        default="/",
    )
    parser.add_argument(
        "--no-check-links",
        action="store_true",
        help="don't check hyperlinks in the generated site",
    )
    parser.add_argument(
        "document_root", metavar="DIRECTORY", help="directory containing source files"
    )
    parser.add_argument("output", metavar="DIRECTORY", help="output directory")
    parser.set_defaults(func=run)



#!/usr/bin/env python3

import os
import sys
from datetime import datetime
from pathlib import Path


# Get command-line arguments
page = Path(sys.argv[1])
basename = sys.argv[2]
file = Path(os.environ["NANCY_INPUT"]) / page.parent / basename

time = os.stat(file).st_mtime
dt = datetime.fromtimestamp(time)
print(dt.strftime("%Y/%m/%d"), end="")



"""Warning and error routines.

Copyright (c) Reuben Thomas 2023.
Released under the GPL version 3, or (at your option) any later version.
"""

import sys
from collections.abc import Callable
from typing import NoReturn, TextIO
from warnings import warn


# Error messages
def simple_warning(prog: str) -> Callable[..., None]:
    def _warning(
        message: Warning | str,
        category: type[Warning],
        filename: str,
        lineno: int,
        file: TextIO | None = sys.stderr,
        line: str | None = None,
    ) -> None:
        print(f"{prog}: {message}", file=file or sys.stderr)

    return _warning


def die(msg: str, code: int | None = 1) -> NoReturn:
    warn(msg)
    sys.exit(code)



#!/usr/bin/env python3
# Output the path from the first argument to the root of the directory

import sys
from pathlib import Path


# Read command-line arguments
page = Path(sys.argv[1])
directory = page.parent

path_to_root = Path(".")
while directory != Path("."):
    directory = directory.parent
    path_to_root /= ".."

print(path_to_root, end="")
